import { ConverterOption } from './converter';
import { ResolvableEngine, ResolvedEngine } from './engine';
type Overwrite<T, U> = Omit<T, Extract<keyof T, keyof U>> & U;
interface IMarpCLIArguments {
    _?: string[];
    allowLocalFiles?: boolean;
    author?: string;
    baseUrl?: string;
    'bespoke.osc'?: boolean;
    'bespoke.progress'?: boolean;
    'bespoke.transition'?: boolean;
    configFile?: string | false;
    description?: string;
    engine?: string;
    html?: boolean;
    image?: string;
    images?: string;
    imageScale?: number;
    inputDir?: string;
    jpegQuality?: number;
    keywords?: string;
    notes?: boolean;
    ogImage?: string;
    output?: string | false;
    pdf?: boolean;
    pdfNotes?: boolean;
    pdfOutlines?: boolean;
    'pdfOutlines.pages'?: boolean;
    'pdfOutlines.headings'?: boolean;
    pptx?: boolean;
    preview?: boolean;
    server?: boolean;
    template?: string;
    theme?: string;
    themeSet?: string[];
    title?: string;
    url?: string;
    watch?: boolean;
}
export type IMarpCLIConfig = Overwrite<Omit<IMarpCLIArguments, '_' | 'configFile' | 'bespoke.osc' | 'bespoke.progress' | 'bespoke.transition' | 'pdfOutlines' | 'pdfOutlines.pages' | 'pdfOutlines.headings'>, {
    bespoke?: {
        osc?: boolean;
        progress?: boolean;
        transition?: boolean;
    };
    engine?: ResolvableEngine;
    html?: ConverterOption['html'];
    keywords?: string | string[];
    lang?: string;
    options?: ConverterOption['options'];
    pdfOutlines?: boolean | {
        pages?: boolean;
        headings?: boolean;
    };
    themeSet?: string | string[];
}>;
export declare class MarpCLIConfig {
    args: IMarpCLIArguments;
    conf: IMarpCLIConfig;
    confPath?: string;
    engine: ResolvedEngine;
    static moduleName: "marp";
    static fromArguments(args: IMarpCLIArguments): Promise<MarpCLIConfig>;
    static isESMAvailable(): boolean;
    private constructor();
    converterOption(): Promise<ConverterOption>;
    get files(): string[];
    private inputDir;
    private loadConf;
    private loadTheme;
}
declare const _default: typeof MarpCLIConfig.fromArguments;
export default _default;
